{% extends "base.html" %}

{% block title %}Application Submitted - {{ config.BBS_NAME }}{% endblock %}

{% block content %}
<div class="row justify-content-center">
    <div class="col-md-7">
        <div class="card">
            <div class="card-header"><i class="bi bi-check-circle text-success"></i> Application Submitted</div>
            <div class="card-body">
                <p>Thanks, {{ req.name }} — your application to join
                   {{ cfg.network_name or 'the network' }} as
                   <strong>{{ req.bbs_name }}</strong> has been submitted
                   and is waiting for the hub sysop to review.</p>
                <p class="text-muted small">You'll hear back at
                   {{ req.email }} once it's been reviewed.</p>
                <a href="{{ url_for('main.index') }}" class="btn btn-primary">
                    <i class="bi bi-house"></i> Back to {{ config.BBS_NAME }}
                </a>
            </div>
        </div>
    </div>
</div>
{% endblock %}
