{% extends "base.html" %}
{% block title %}Resend Verification Email{% endblock %}
{% block content %}
<div class="row justify-content-center mt-4">
<div class="col-md-5">
    <div class="card p-4">
        <h2><i class="bi bi-envelope-arrow-up"></i> Resend Verification</h2>
        <p class="text-muted">Enter your username/email and password to receive a new verification link.</p>
        {% if error %}<div class="alert alert-danger">{{ error }}</div>{% endif %}
        <form method="post">
            <input type="hidden" name="csrf_token" value="{{ csrf_token() }}">
            <div class="mb-3">
                <label class="form-label">Username or Email</label>
                <input name="identifier" class="form-control" autofocus required>
            </div>
            <div class="mb-3">
                <label class="form-label">Password</label>
                <input name="password" type="password" class="form-control" required>
            </div>
            <button class="btn btn-primary w-100"><i class="bi bi-send"></i> Send Link</button>
        </form>
        <div class="text-center mt-3 small">
            <a href="{{ url_for('auth.login') }}">Back to login</a>
        </div>
    </div>
</div>
</div>
{% endblock %}
