#ifndef ANET_DROPFILE_H
#define ANET_DROPFILE_H

#include "common.h"
#include "config.h"

typedef struct {
    int active;
    char sysop[64];
    char user_name[64];
    char alias[64];
    char location[64];
    char com_type[32];
    int node;
} dropfile_info_t;

int dropfile_detect(const char *explicit_path, char *found, size_t found_sz);
int dropfile_load(const char *path, dropfile_info_t *out, bookmark_t *bm);

#endif
