{% extends "base.html" %}
{% block title %}Awaiting Approval{% endblock %}
{% block content %}
<div class="row justify-content-center mt-4">
<div class="col-md-6">
    <div class="card text-center p-4">
        <div class="mb-3" style="font-size:3rem;">⏳</div>
        <h2>Account created, {{ username }}!</h2>
        <p class="text-muted mb-3">
            New accounts on this BBS require sysop approval before you can log in.
        </p>
        <p class="text-muted small">
            You'll be able to log in once the sysop has reviewed your registration.
        </p>
        <hr>
        <p class="small">
            <a href="{{ url_for('auth.login') }}">Back to login</a>
        </p>
    </div>
</div>
</div>
{% endblock %}
