#ifndef COLORS_H
#define COLORS_H

// Color codes
#define COLOR_BLACK     0
#define COLOR_RED       1
#define COLOR_GREEN     2
#define COLOR_YELLOW    3
#define COLOR_BLUE      4
#define COLOR_MAGENTA   5
#define COLOR_CYAN      6
#define COLOR_WHITE     7

// IRC color codes (mIRC colors)
#define IRC_WHITE       0
#define IRC_BLACK       1
#define IRC_BLUE        2
#define IRC_GREEN       3
#define IRC_RED         4
#define IRC_BROWN       5
#define IRC_MAGENTA     6
#define IRC_ORANGE      7
#define IRC_YELLOW      8
#define IRC_LIGHT_GREEN 9
#define IRC_CYAN        10
#define IRC_LIGHT_CYAN  11
#define IRC_LIGHT_BLUE  12
#define IRC_LIGHT_MAGENTA 13
#define IRC_GRAY        14
#define IRC_LIGHT_GRAY  15

const char *color_get_ansi(int fg, int bg, int dark_mode);
const char *color_irc_to_ansi(int irc_color);

#endif // COLORS_H