{% extends "base.html" %}
{% block title %}Verify Your Email{% endblock %}
{% block content %}
<div class="row justify-content-center mt-4">
<div class="col-md-6">
    <div class="card text-center p-4">
        <div class="mb-3" style="font-size:3rem;">📧</div>
        <h2>Check your email</h2>
        <p class="text-muted mb-3">
            We sent a verification link to <strong>{{ email }}</strong>.<br>
            Click the link in that email to activate your account.
        </p>
        <p class="text-muted small">
            The link expires in <strong>24 hours</strong>. If you don't see it, check your spam folder.
        </p>
        <hr>
        <p class="small">
            Didn't receive it?
            <a href="{{ url_for('auth.resend_verification') }}">Resend verification email</a>
        </p>
    </div>
</div>
</div>
{% endblock %}
