{% extends "base.html" %}
{% block title %}Email Verified{% endblock %}
{% block content %}
<div class="row justify-content-center mt-4">
<div class="col-md-6">
    <div class="card text-center p-4">
        <div class="mb-3" style="font-size:3rem;">✅</div>
        <h2>Email Verified!</h2>
        <p class="text-muted mb-3">
            Your email address has been confirmed, <strong>{{ username }}</strong>.<br>
            You're now logged in and ready to go.
        </p>
        <a href="{{ url_for('main.index') }}" class="btn btn-primary">
            <i class="bi bi-house"></i> Go to the BBS
        </a>
    </div>
</div>
</div>
{% endblock %}
