#ifndef ANET_CONFIG_H
#define ANET_CONFIG_H

#include "common.h"

typedef struct {
    char label[64];
    char server[128];
    int port;
    char nick[64];
    char channel[64];
    int tls;
    char password[128];   /* SASL PLAIN / NickServ password */
} bookmark_t;

typedef struct {
    bookmark_t items[CFG_MAX_BOOKMARKS];
    int count;
    char path[260];
} config_t;

void config_init(config_t *cfg);
void config_load(config_t *cfg);
void config_save(config_t *cfg);
void config_add_or_update(config_t *cfg, const bookmark_t *bm);
void config_delete(config_t *cfg, int idx);

#endif
